require_relative '../contraption'

module Contraption
  class Catalog
    include Enumerable

    attr_reader :items

    def initialize items
      @items = Array items
    end

    def << new_item
      @items.concat(Array new_item)
    end

    def by_year
      by_date { |date| Year.new date.year }
    end

    def by_month
      by_date { |date| Month.new date.year, date.month }
    end

    def by_day
      by_date { |date| Day.new date.year, date.month, date.day }
    end

    def by_tag
      tags = all(:tags).uniq{|t| t.to_url}
      items.each_with_object(Hash.new {|h,k| h[k] = []}) do |item, result|
        item.tags.each{|t| result[t.to_url] << item}
      end
    end

    def each
      items.each
    end

    def most_recent n=1
      n = items.length if n < 0
      items.sort_by{ |item| item.publication_date }
           .reverse
           .take n
    end

    private
    def by &block
      items.group_by do |item|
        yield item
      end
    end

    def by_date &block
      by { |i| yield block.call(i.publication_date) }
    end

    def all attribute
      items.map{|item| item.public_send(attribute.to_sym)}
           .flatten
           .uniq
    end
  end
end
