require 'date'
require_relative 'tag'

module Contraption
  class Header
    class << self
      def from string
        Header.new (Hash[
            string.lines
                  .map{|l| process l}
                  .reject{|l| l == nil}
        ])
      end

      private
        def process line
          return nil if /^=+$/.match line
          splitted = line.strip.split /^(\w+):\s+/, 2
          return [:title, line.strip.chomp] if splitted.count == 1

          key = splitted[1].downcase.to_sym
          raw_value = splitted[2].strip
          case key
          when :external
            [:external, raw_value]
          when :filename
            [:filename, raw_value]
          when :published
            [:publication_date, determine_date(raw_value)]
          when :summary
            [:summary, raw_value]
          when :tags
            [:tags, raw_value.split(/,\s+/).map{|t| Tag.new t}]
          when :type
            [:type, raw_value.downcase.to_sym]
          when :title
            [:title, raw_value]
          else
            nil
          end
        end

        def determine_date raw_value
          if raw_value.downcase == "now"
            :now
          else
            DateTime.parse(raw_value)
          end
        end
    end

    def initialize opts={}
      @defaults = { title: "",
                    external: nil,
                    publication_date: "",
                    summary: "",
                    type: "",
                    tags: []}.merge!(opts)
    end

    def filename ext='.md'
      fn = @defaults.fetch(:filename) do
        title.downcase
             .gsub(/[^\w\s-]+/, ' ')
             .strip
             .gsub(/\s+/, '-') + ext
      end.split ext
      fn.join + ext
    end

    def update new_opts={}
      Header.new @defaults.merge(new_opts)
    end

    def new?
      publication_date == :now
    end

    private
      def method_missing m, *a, &b
        @defaults.fetch(m) { super }
      end
  end
end
