require_relative '../contraption'
require 'pathname'

module Contraption
  class Location
    def initialize path="."
      @pn = Pathname.new path
    end

    def list ext=/.*/
      ext = Array(ext).join('|')
      ext = Regexp.new ext
      pn.find
        .reject { |f| f == pn }
        .select { |f| ext.match f.extname }
        .map{ |f| f.basename.to_s }
    end

    def entries
      pn.entries.map{|e| e.expand_path(pn) }
    end

    def path
      pn.to_s
    end

    def to_s
      pn.expand_path.to_s
    end

    def read file_name
      complete_name = pn+file_name
      return :file_does_not_exist unless complete_name.exist?
      return :file_not_readable unless complete_name.readable?

      return complete_name.read unless block_given?
      complete_name.open("r") { |f| yield f }
    end

    def destroy
      FileUtils.rm_rf pn.expand_path
    end

    def cd directory
      l = Location.new(pn+directory)
      l.create!  unless l.pn.directory?
      l
    end

    def create!
      pn.mkpath unless exist?
    end

    def remove file_name
      complete_name = pn+file_name
      return :file_does_not_exist unless complete_name.exist?

      complete_name.delete
    end

    def write file_name, content
      complete_name = pn+file_name
      return :file_already_exists if complete_name.exist?

      complete_name.dirname.mkpath unless complete_name.dirname.exist?

      complete_name.open("w:UTF-8") {|f| f.print content}
    end

    def == other_location
      path == other_location.path
    end

    def exist?
      pn.exist?
    end

    def readable?
      pn.readable?
    end

    def writable?
      pn.writable?
    end

    def executable?
      pn.executable?
    end

    protected
      attr_reader :pn
  end
end
