require_relative 'repository'
require_relative 'site'
require_relative 'http_handler'

module Contraption
  class Orchestrator
    def initialize options
      @options = options
    end

    def run!
      source.finalize_completed_drafts handlers
      site.build_all_individuals
      site.build_month_pages
      site.build_year_pages
      site.build_tag_pages
      site.build_tag_cloud
      site.build_recent
      site.build_landing
      site.build_rss
      site.build_archive_navigation
      copy_static_files

      post_run
    end

    private
    def site
      @site ||= Site.new @options.destination, posts, formatter
    end

    def source
      raise "Not implemented"
    end

    def post_run; end

    def formatter
      @formatter ||= source.formatter
    end

    def posts
      @posts ||= source.posts
    end

    def copy_static_files
      `cp -r #{source.static_file_path}/* #{site.root}` unless (Dir[source.static_file_path.to_s + "/*"]).length == 0
    end

    def handlers
      [ HttpHandler.new ]
    end
  end
end
