require_relative 'header'

require 'redcarpet'

module Contraption
  class Post
    class << self
      def build data=""
        data = data.to_s

        splitted = data.split(/\n\n/, 2)
        Post.new Header.from(splitted.first), translate_markdown(splitted.last)
      rescue ArgumentError => e
        Post.new Header.from(""), ""
      end

      def translate_markdown content
        Redcarpet::Markdown.new(Redcarpet::Render::HTML).render(content)
      end
    end

    def publish link_handlers=[]
      links = list_links

      links.map! do |link|
        h = link_handlers.select{|l| l.protocol == link.first.to_sym}
        if h.length > 1
          raise "Multiple link handlers for #{link.first} protocol"
        elsif h.length == 0
          nil
        else
          h = h.first
          [link, h.handle(link)]
        end
      end.compact!

      update_links links

      Post.new( metadata.update({publication_date: Time.now}), body)
    end

    attr_reader :body, :metadata
    private
      def initialize  metadata="", body=""
        @body = body
        @metadata = metadata
      end

      def method_missing m, *a, &b
        metadata.public_send(m, *a, &b)
      end

      def list_links
        links = []
        body.scan(/="([^"]*)"/) do |link|
          links << link.first.split('://', 2)
        end
        links
      end

      def update_links new_links
        new_links.each do |link|
          orig = link.first.join('://')
          new  = link.last.join('://')
          @body.gsub!("=\"#{orig}\"", "=\"#{new}\"")
        end
      end
  end
end
