require_relative "post"
require_relative "formatter"
require_relative "catalog"

module Contraption
  class Repository
    attr_reader :formatter

    def initialize path
      @path = path
      validate
      construct_locations
      @formatter = Formatter.new @formats_source
    end

    def clone
      new_path = @path.cd "../.tmp_#{Time.now.to_i}"
      `cp -r #{@path.path}/* #{new_path.path}`
      Repository.new new_path
    end

    def destroy
      @path.destroy
    end

    def validate
      raise "Invalid source directory structure in #{@path}" unless valid_layout?
    end

    def valid_layout?
      @path.exist? and @path.list.include? "drafts" and @path.list.include? "posts"
    end

    def drafts
      @drafts_source.list
    end

    def posts
      Catalog.new( @posts_source.list.map do |p|
        begin
          Post.build(@posts_source.read p)
        rescue ArgumentError => e
          STDERR.puts "Invalid post #{p}"
        end
      end.select{|p| p.title != ''}
      )
    end

    def static_file_path
      @static_source.path
    end

    def static_files
      @static_source.entries
    end

    def completed_drafts
      drafts.map{|draft| [draft, @drafts_source.read(draft)]}
            .each_with_object([]) do |draft, result|
              result << draft.first if Post.build(draft.last).new?
            end
    end

    def finalize_completed_drafts link_handlers=[]
      completed_drafts.map{|draft| [draft, Post.build(@drafts_source.read(draft)).publish(link_handlers)]}
                      .each do |filename, post|
                        @posts_source.write post.publication_date.strftime("%Y%m%d-")+post.filename, @formatter.format(post, :raw)
                        @drafts_source.remove filename
                      end
    end

    def construct_locations
      @drafts_source  = @path.cd("drafts")
      @posts_source   = @path.cd("posts")
      @formats_source = @path.cd("formats")
      @static_source  = @path.cd("static")
    end
  end
end
